function checkContactsAccess()
  local activity = this
  local resolver = activity.getContentResolver()
  local hasPermission = activity.checkSelfPermission("android.permission.READ_CONTACTS") == activity.PackageManager.PERMISSION_GRANTED
  if not hasPermission then
    activity.requestPermissions({"android.permission.READ_CONTACTS"}, 1)
    return "Permission required to access contacts."
  end
  local cursor = resolver.query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, {ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME, ContactsContract.CommonDataKinds.Phone.NUMBER}, nil, nil, nil)
  local contactsList = {}
  if cursor then
    while cursor.moveToNext() do
      local name = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME))
      local number = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
      contactsList[normalizeNumber(number)] = name -- تخزين الرقم بدون مسافات
    end
    cursor.close()
   else
    return "Failed to retrieve contacts."
  end
  return contactsList
end

function getCallLog(contactsList)
  local activity = this
  local resolver = activity.getContentResolver()
  local callLogUri = CallLog.Calls.CONTENT_URI
  local cursor = resolver.query(callLogUri, {CallLog.Calls.NUMBER, CallLog.Calls.TYPE, CallLog.Calls.DATE, CallLog.Calls.DURATION}, nil, nil, CallLog.Calls.DATE .. " DESC")
  local resultsList = {}
  if cursor then
    while cursor.moveToNext() do
      local number = normalizeNumber(cursor.getString(cursor.getColumnIndex(CallLog.Calls.NUMBER)))
      local type = cursor.getString(cursor.getColumnIndex(CallLog.Calls.TYPE))
      local date = cursor.getString(cursor.getColumnIndex(CallLog.Calls.DATE))
      local duration = tonumber(cursor.getString(cursor.getColumnIndex(CallLog.Calls.DURATION))) -- الحصول على مدة المكالمة وتحويلها إلى رقم

      local typeString = ""
      if type == tostring(CallLog.Calls.INCOMING_TYPE) then
        typeString = "incoming"
       elseif type == tostring(CallLog.Calls.OUTGOING_TYPE) then
        typeString = "outgoing"
       elseif type == tostring(CallLog.Calls.MISSED_TYPE) then
        typeString = "missed"
       elseif type == tostring(CallLog.Calls.REJECTED_TYPE) then
        typeString = "Rejected"
      end

      local dateNumber = tonumber(date)
      local seconds = math.floor(dateNumber / 1000)
      local formattedDate = os.date("%d-%m-%Y %H:%M", seconds)
      local contactName = contactsList[number] or "unknown"

      local hours = math.floor(duration / 3600)
      local minutes = math.floor((duration % 3600) / 60)
      local secs = duration % 60
      local formattedDuration = string.format("%02d:%02d:%02d", hours, minutes, secs)

      table.insert(resultsList, {
        name = contactName,
        type = typeString,
        date = formattedDate,
        number = number,
        duration = formattedDuration
      })
    end
    cursor.close()
   else
    print("لا يوجد سجل مكالمات.")
  end
  return resultsList
end

function formatResults(results)
  local formatted = {}
  if results then
    for _, result in ipairs(results) do
      local entry = result.name .. ", \n" .. result.type .. ", \n" .. result.number .. ". \n" .. result.date .. ". \n" .. result.duration
      table.insert(formatted, entry)
    end
  end
  return formatted
end


function normalizeNumber(number)
  return number:gsub("%s+", "") -- إزالة المسافات
end



function printOverallCallStatistics(resultsList)
  local totalCalls = 0
  local outgoingCalls = 0
  local incomingCalls = 0
  local rejectedCalls = 0
  local missedCalls = 0
  local unknownCalls = 0
  local totalDuration = 0
  local totalOutgoingDuration = 0
  local totalIncomingDuration = 0
  for _, result in ipairs(resultsList) do
    totalCalls = totalCalls + 1
    local hours, minutes, seconds = result.duration:match("(%d+):(%d+):(%d+)")
    local durationInSeconds = tonumber(hours) * 3600 + tonumber(minutes) * 60 + tonumber(seconds)
    totalDuration = totalDuration + durationInSeconds
    if result.type == "outgoing" then
      outgoingCalls = outgoingCalls + 1
      totalOutgoingDuration = totalOutgoingDuration + durationInSeconds
     elseif result.type == "incoming" then
      incomingCalls = incomingCalls + 1
      totalIncomingDuration = totalIncomingDuration + durationInSeconds
     elseif result.type == "Rejected" then
      rejectedCalls = rejectedCalls + 1
     elseif result.type == "missed" then
      missedCalls = missedCalls + 1 -- تحديث عدد المكالمات الفائتة
    end
    if result.name == "unknown" or result.number == "" then
      unknownCalls = unknownCalls + 1
    end
  end
  local function formatDuration(duration)
    local hours = math.floor(duration / 3600)
    local minutes = math.floor((duration % 3600) / 60)
    local seconds = duration % 60
    return string.format("%02d:%02d:%02d", hours, minutes, seconds)
  end
  print("Call Count:")
  print("total: " .. totalCalls)
  print("outgoing: " .. outgoingCalls)
  print("incoming: " .. incomingCalls)
  print("missed: " .. missedCalls)
  print("rejected: " .. rejectedCalls)
  print("unknown: " .. unknownCalls)
  print("Call Duration")
  print("total : " .. formatDuration(totalDuration))
  print("Outgoing : " .. formatDuration(totalOutgoingDuration))
  print("Incoming: " .. formatDuration(totalIncomingDuration))
end



function printCallStatistics(resultsList)
  local callStats = {}
  for _, result in ipairs(resultsList) do
    local identifier = result.name ~= "unknown" and result.name or result.number -- استخدام الاسم أو الرقم إذا لم يكن الاسم موجودًا
    if not callStats[identifier] then
      callStats[identifier] = {count = 0, totalDuration = 0}
    end
    callStats[identifier].count = callStats[identifier].count + 1
    local hours, minutes, seconds = result.duration:match("(%d+):(%d+):(%d+)")
    local durationInSeconds = tonumber(hours) * 3600 + tonumber(minutes) * 60 + tonumber(seconds)
    callStats[identifier].totalDuration = callStats[identifier].totalDuration + durationInSeconds
  end
  local sortedStats = {}
  for identifier, stats in pairs(callStats) do
    table.insert(sortedStats, {name = identifier, count = stats.count, totalDuration = stats.totalDuration})
  end
  table.sort(sortedStats, function(a, b)
    return a.count > b.count
  end)
  print("Call Statistics by Contact")
  for _, entry in ipairs(sortedStats) do
    local hours = math.floor(entry.totalDuration / 3600)
    local minutes = math.floor((entry.totalDuration % 3600) / 60)
    local seconds = entry.totalDuration % 60
    local formattedDuration = string.format("%02d:%02d:%02d", hours, minutes, seconds)
    print(entry.name .. ", " .. entry.count .. ", " .. formattedDuration)
  end
end




function getAllCallLogs(resultsList)
  local contactsList = checkContactsAccess()
  local resultsList = getCallLog(contactsList)
  local activity = this
  local layout = LinearLayout(activity)
  layout.setOrientation(LinearLayout.VERTICAL)
  layout.setBackgroundColor(0xFF000000) -- لون أسود

  local titleLayout = LinearLayout(activity)
  titleLayout.setOrientation(LinearLayout.HORIZONTAL)
  local allCallsText = TextView(activity)
  allCallsText.setText("All Calls")
  allCallsText.setTextSize(30)
  allCallsText.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {
    onClick = function(view)
      service.click({{"%返回"}})
    end
  }))
  allCallsText.setOnLongClickListener(luajava.createProxy("android.view.View$OnLongClickListener", {
    onLongClick = function(view)
      service.click({{"%返回"}})
      task(200, function()
        displayContacts()
      end)
      return true -- يجب إرجاع true للإشارة إلى أن الحدث تم التعامل معه
    end
  }))
  local paramsTitle = LinearLayout.LayoutParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1)
  allCallsText.setLayoutParams(paramsTitle)
  titleLayout.addView(allCallsText)
  layout.addView(titleLayout)

  local callCount = TextView(activity)
  local totalCalls = #resultsList
  callCount.setText("Item Count: " .. totalCalls)
  callCount.setTextSize(23)
  callCount.setOnClickListener(luajava.createProxy("android.view.View$OnClickListener", {
    onClick = function(view)
      printOverallCallStatistics(resultsList)
      printCallStatistics(resultsList)
    end
  }))

  layout.addView(callCount)

  searchBox = EditText(activity)
  searchBox.contentDescription = "Search by name, number, type, or date"
  searchBox.setBackgroundColor(0xFF006400)
  layout.addView(searchBox)

  local buttonLayout = LinearLayout(activity)
  buttonLayout.setOrientation(LinearLayout.HORIZONTAL)
  local buttons = {
    {desc = "All", filter = function() return resultsList end, color = 0xFF795548},
    {desc = "outgoing", filter = function() return filterCalls("outgoing") end, color = 0xFFD84315},
    {desc = "incoming", filter = function() return filterCalls("incoming") end, color = 0xFF388E3C},
    {desc = "missed", filter = function() return filterCalls("missed") end, color = 0xFFFF8F00},
    {desc = "Rejected", filter = function() return filterCalls("Rejected") end, color = 0xFFB71C1C},
    {desc = "unknown", filter = function() return filterCalls("unknown") end, color = 0xFF424242}
  }

  for _, btn in ipairs(buttons) do
    local button = Button(activity)
    button.contentDescription = btn.desc
    button.setTextSize(10)
    button.setLayoutParams(LinearLayout.LayoutParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1))
    button.setBackgroundColor(btn.color)
    button.onClick = function()
      local filteredResultsList = btn.filter()
      updateListView(filteredResultsList)
      callCount.setText("Item Count: " .. #filteredResultsList)
      allCallsText.setText(btn.desc)
    end
    buttonLayout.addView(button)
  end

  layout.addView(buttonLayout)
  local listView = ListView(activity)
  layout.addView(listView)
  local filteredResultsList = resultsList
  local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, formatResults(filteredResultsList))
  listView.setAdapter(adapter)

  local dlg = LuaDialog(activity).setView(layout)
  dlg.show()

  searchBox.addTextChangedListener(luajava.createProxy("android.text.TextWatcher", {
    onTextChanged = function(s, start, before, count)
      local query = tostring(s):lower()
      local filteredResultsList = {}
      local parts = {}
      for part in query:gmatch("[^*]+") do
        table.insert(parts, part:match("^%s*(.-)%s*$"))
      end
      for _, result in ipairs(resultsList) do
        local dateOnly = result.date:match("(%d%d%-%d%d%-%d%d%d%d)")
        local matches = true
        for _, part in ipairs(parts) do
          if part:match("^%d%d%-%d%d%-%d%d%d%d$") then
            local day, month, year = part:match("(%d%d)%-(%d%d)%-(%d%d%d%d)")
            if not (dateOnly:find(year) and dateOnly:find(month) and dateOnly:find(day)) then
              matches = false
              break
            end
           elseif normalizeNumber(result.number):find(part) or result.name:lower():find(part) or result.type:lower():find(part) then
           else
            matches = false
            break
          end
        end
        if matches then
          table.insert(filteredResultsList, result)
        end
      end
      updateListView(filteredResultsList)
    end,
  }))



  listView.setOnItemClickListener(luajava.createProxy("android.widget.AdapterView$OnItemClickListener", {
    onItemClick = function(parent, view, position, id)
      local fullText = tostring(view.getText()) -- جلب النص الكامل من `view`
      local number = fullText:match("(%d+[%d%s%-]*)") -- استخراج الرقم من النص
      if number then
        local i = Intent(Intent.ACTION_CALL, Uri.parse("tel:" .. Uri.encode(number)))
        activity.startActivity(i)
        dlg.dismiss()
       else
        service.speak("يُرْجَى اختيارُ الرقم الصحيح.")
      end
    end
  }))



  listView.setOnItemLongClickListener(luajava.createProxy("android.widget.AdapterView$OnItemLongClickListener", {
    onItemLongClick = function(parent, view, position, id)


      local subMenuItems = {
        "Select SIM card",
        "Send SMS",
        "Show history",
        "Show contact info",
        "Add number",
        "Edit contact info",
        "Copy number",
        "Copy contact info",
        "Share number",
        "Share contact info",
      }


      layout = {
        GridView,
        id = "grid",
        numColumns = 2, -- عدد الأعمدة في الشبكة
        background = "#ff00002e", -- لون الخلفية
        layout_width = "fill",
        layout_height = "fill"
      }


      dlg = LuaDialog(service)
      service.postSpeak(200,"Choose an option")
      dlg.View = loadlayout(layout)
      grid.adapter = SingleLineAdapter(service, String(subMenuItems))
      grid.onItemClick = function(_, _, position)
        dlg.dismiss()

        if position == 0 then
          local fullText = tostring(view.getText()) -- جلب النص الكامل من `view`
          local number = fullText:match("(%d+[%d%s%-]*)") -- استخراج الرقم من النص
          if number then
            local i = Intent(Intent.ACTION_DIAL, Uri.parse("tel:" .. Uri.encode(number)))
            activity.startActivity(i)
            service.click({{"%返回"}})
           else
            service.speak("يُرْجَى اختيارُ الرقم الصحيح.")
          end

         elseif position == 1 then
          local fullText = tostring(view.getText()) -- جلب النص الكامل من `view`
          local number = fullText:match("(%d+[%d%s%-]*)") -- استخراج الرقم من النص
          if number then
            local smsIntent = Intent(Intent.ACTION_SENDTO, Uri.parse("smsto:" .. Uri.encode(number)))
            smsIntent.putExtra("sms_body", "السلام عليكم\n")
            activity.startActivity(smsIntent)
            service.click({{"%返回"}})
           else
            service.speak("يُرْجَى اختيارُ الرقم الصحيح.")
          end

         elseif position == 2 then
          local text = tostring(view.getText()) -- جلب النص الكامل من `view`
          local lines = {}
          for line in text:gmatch("([^\n]*)\n?") do
            table.insert(lines, line)
          end
          if #lines >= 3 then
            local line3 = lines[3]
            line3 = line3:gsub("(%d+)[.,]%s*", "%1") -- استبدال الرقم متبوعًا بفاصلة أو نقطة ومسافة
            task(200, function()
              service.setText(line3)
            end)
           else
            this.postSpeak(200, "النص يحتوي على أقل من ثلاثة أسطر.")
          end

         elseif position == 3 then
          local text = tostring(view.getText())

          local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول

          contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
          if not contactName then
            Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
            return
          end
          local contentResolver = activity.getContentResolver()

          local cursor = contentResolver.query(
          ContactsContract.Contacts.CONTENT_URI,
          { ContactsContract.Contacts._ID },
          ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
          { contactName },
          nil
          )
          if cursor ~= nil and cursor.moveToFirst() then

            local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            cursor.close()

            local viewIntent = Intent(Intent.ACTION_VIEW)
            viewIntent.setData(Uri.withAppendedPath(ContactsContract.Contacts.CONTENT_URI, contactId))
            activity.startActivity(viewIntent)
            service.click({{"%返回"}})
           else

            Toast.makeText(activity, "Contact not found", Toast.LENGTH_SHORT).show()
          end

         elseif position == 4 then
          local text = tostring(view.getText())
          local lines = {}
          for line in text:gmatch("([^\n]*)\n?") do
            table.insert(lines, line)
          end
          if #lines >= 3 then
            local line3 = lines[3]
            line3 = line3:gsub("(%d+)[.,]%s*", "%1") -- استبدال الرقم متبوعًا بفاصلة أو نقطة ومسافة
            intent = Intent(Intent.ACTION_INSERT_OR_EDIT)
            intent.setType(ContactsContract.Contacts.CONTENT_ITEM_TYPE)
            intent.putExtra(ContactsContract.Intents.Insert.PHONE, line3)
            this.startActivity(intent)
            service.click({{"%返回"}})
           else
            this.postSpeak(200, "النص يحتوي على أقل من ثلاثة أسطر.")
          end

         elseif position == 5 then
          local text = tostring(view.getText())

          local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول

          contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
          if not contactName then
            Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
            return
          end
          local contentResolver = activity.getContentResolver()

          local cursor = contentResolver.query(
          ContactsContract.Contacts.CONTENT_URI,
          { ContactsContract.Contacts._ID },
          ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
          { contactName },
          nil
          )
          if cursor ~= nil and cursor.moveToFirst() then

            local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            cursor.close()

            local editIntent = Intent(Intent.ACTION_EDIT)
            editIntent.setData(Uri.withAppendedPath(ContactsContract.Contacts.CONTENT_URI, contactId))
            editIntent.putExtra("finishActivityOnSaveCompleted", true)
            activity.startActivity(editIntent)
            service.click({{"%返回"}})
           else

            Toast.makeText(activity, "Contact not found", Toast.LENGTH_SHORT).show()
          end

         elseif position == 6 then
          local text = tostring(view.getText())
          local lines = {}
          for line in text:gmatch("([^\n]*)\n?") do
            table.insert(lines, line)
          end
          if #lines >= 3 then
            local line3 = lines[3]
            line3 = line3:gsub("(%d+)[.,]%s*", "%1") -- استبدال الرقم متبوعًا بفاصلة أو نقطة ومسافة
            service.copy(line3)
            this.postSpeak(200, "copy: ".. line3)
           else
            this.postSpeak(200, "النص يحتوي على أقل من ثلاثة أسطر.")
          end

         elseif position == 7 then
          local text = tostring(view.getText())

          local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول

          contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
          if not contactName then
            Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
            return
          end
          local contentResolver = activity.getContentResolver()

          local cursor = contentResolver.query(
          ContactsContract.Contacts.CONTENT_URI,
          { ContactsContract.Contacts._ID, ContactsContract.Contacts.HAS_PHONE_NUMBER },
          ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
          { contactName },
          nil
          )
          if cursor ~= nil and cursor.moveToFirst() then
            local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            local hasPhoneNumber = cursor.getInt(cursor.getColumnIndex(ContactsContract.Contacts.HAS_PHONE_NUMBER))
            cursor.close()
            local contactInfo = contactName
            if hasPhoneNumber > 0 then

              local phoneCursor = contentResolver.query(
              ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
              { ContactsContract.CommonDataKinds.Phone.NUMBER },
              ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?",
              { contactId },
              nil
              )
              local phoneNumbers = {}
              local uniqueNumbers = {} -- تخزين الأرقام الفريدة فقط
              while phoneCursor ~= nil and phoneCursor.moveToNext() do

                local phoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER)):gsub("%s+", "")

                if not uniqueNumbers[phoneNumber] then
                  uniqueNumbers[phoneNumber] = true -- علامة على وجود الرقم
                  table.insert(phoneNumbers, phoneNumber) -- إضافة الرقم بعد التأكد من عدم التكرار
                end
              end
              phoneCursor.close()

              contactInfo = contactInfo .. "\n" .. table.concat(phoneNumbers, "\n")
             else
              contactInfo = contactInfo .. "\nNo phone number found for this contact."
            end

            service.copy(contactInfo)
            Toast.makeText(activity, "Contact info copied to clipboard.", Toast.LENGTH_SHORT).show()
            service.click({{"%返回"}})
           else

            Toast.makeText(activity, "Contact not found.", Toast.LENGTH_SHORT).show()
          end

         elseif position == 8 then
          local text = tostring(view.getText())
          local lines = {}
          for line in text:gmatch("([^\n]*)\n?") do
            table.insert(lines, line)
          end
          if #lines >= 3 then
            local line3 = lines[3]
            line3 = line3:gsub("(%d+)[.,]%s*", "%1") -- استبدال الرقم متبوعًا بفاصلة أو نقطة ومسافة
            local intent = Intent(Intent.ACTION_SEND)
            .setType("text/plain")
            .putExtra(Intent.EXTRA_TEXT, line3) -- مشاركة السطر الثالث فقط
            this.startActivity(Intent.createChooser(intent, "مشاركة عبر:"))
            service.click({{"%返回"}})
           else
            this.postSpeak(200, "النص يحتوي على أقل من ثلاثة أسطر.")
          end

         elseif position == 9 then
          local text = tostring(view.getText())

          local contactName = text:match("^[^\n]+") -- هذا التعبير سيأخذ النص في السطر الأول

          contactName = contactName:gsub("[.,]%s*$", "") -- إزالة الفاصلة أو النقطة المتبوعة بمسافة
          if not contactName then
            Toast.makeText(activity, "Contact name not found in the first line of the text.", Toast.LENGTH_SHORT).show()
            return
          end
          local contentResolver = activity.getContentResolver()

          local cursor = contentResolver.query(
          ContactsContract.Contacts.CONTENT_URI,
          { ContactsContract.Contacts._ID, ContactsContract.Contacts.HAS_PHONE_NUMBER },
          ContactsContract.Contacts.DISPLAY_NAME .. " = ?",
          { contactName },
          nil
          )
          if cursor ~= nil and cursor.moveToFirst() then
            local contactId = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            local hasPhoneNumber = cursor.getInt(cursor.getColumnIndex(ContactsContract.Contacts.HAS_PHONE_NUMBER))
            cursor.close()
            local contactInfo = contactName
            if hasPhoneNumber > 0 then

              local phoneCursor = contentResolver.query(
              ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
              { ContactsContract.CommonDataKinds.Phone.NUMBER },
              ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?",
              { contactId },
              nil
              )
              local phoneNumbers = {}
              local uniqueNumbers = {} -- تخزين الأرقام الفريدة فقط
              while phoneCursor ~= nil and phoneCursor.moveToNext() do

                local phoneNumber = phoneCursor.getString(phoneCursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER)):gsub("%s+", "")

                if not uniqueNumbers[phoneNumber] then
                  uniqueNumbers[phoneNumber] = true -- علامة على وجود الرقم
                  table.insert(phoneNumbers, phoneNumber) -- إضافة الرقم بعد التأكد من عدم التكرار
                end
              end
              phoneCursor.close()

              contactInfo = contactInfo .. "\n" .. table.concat(phoneNumbers, "\n")
             else
              contactInfo = contactInfo .. "\nNo phone number found for this contact."
            end

            local sendIntent = Intent(Intent.ACTION_SEND)
            sendIntent.setType("text/plain")
            sendIntent.putExtra(Intent.EXTRA_TEXT, contactInfo)
            activity.startActivity(Intent.createChooser(sendIntent, "Send Contact Info"))
            service.click({{"%返回"}})
           else

            Toast.makeText(activity, "Contact not found.", Toast.LENGTH_SHORT).show()
          end


        end
      end

      -- وظيفة الضغط المطول على العناصر في الشبكة
      grid.onItemLongClick = function(_, _, position)
        dlg.dismiss()

        if position == 0 then

         elseif position == 1 then

         elseif position == 2 then

         elseif position == 3 then

         elseif position == 4 then

         elseif position == 5 then

         elseif position == 6 then

         elseif position == 7 then
        end
      end

      -- إظهار الشبكة عند تشغيل التطبيق
      dlg.show()
      return true
    end
  }))

  function updateListView(filteredResults)
    adapter.clear()

    -- تحقق مما إذا كان حقل البحث فارغًا
    if tostring(searchBox.getText()):match("^%s*$") then

      filteredResults = filterUniqueCallsByContact(filteredResults)
    end

    adapter.addAll(formatResults(filteredResults))
    adapter.notifyDataSetChanged()

    callCount.setText("Item Count: " .. #filteredResults)

    if #filteredResults > 0 then
      local firstResult = filteredResults[1]
      task(100, function()
        service.appendSpeak2(#filteredResults .. ". " .. firstResult.name .. ", " .. firstResult.type .. ", " .. firstResult.number .. ", " .. firstResult.date)
      end)
     else
      task(100, function()
        service.appendSpeak2("غير مطابق")
      end)
    end
  end

  -- دالة لتصفية المكالمات بحيث تظهر عنصر واحد فقط لكل جهة اتصال
  function filterUniqueCallsByContact(callList)
    local uniqueResults = {}
    local seenContacts = {}

    for _, call in ipairs(callList) do
      local identifier = call.name ~= "unknown" and call.name or call.number
      if not seenContacts[identifier] then
        table.insert(uniqueResults, call)
        seenContacts[identifier] = true
      end
    end

    return uniqueResults
  end



  function filterCalls(callType)
    local filteredResults = {}
    for _, result in ipairs(resultsList) do
      if result.type == callType or (callType == "unknown" and result.name == "unknown") then
        table.insert(filteredResults, result)
      end
    end
    return filteredResults
  end
end